#!/usr/bin/python
# -*- coding: utf-8 -*-

# cli.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>


# Inclusions des modules:
# Natifs
import os, platform
import re
import time

# Perso
from commonFunctions import *
from download import *

# Les differentes plateforme
import Megaupload
import Youtube
import Classic

#############################################################################
class Cli():

	#========================================================================
	def __init__(self):
		##########################################################
		#                Default Var Initialisation              #
		##########################################################
		
		self.appMethod					= "cli"
		self.dwnArg						= ""
		self.configDir					= getConfDir()
		self.execDir					= getExecDir()
		self.userPersonalPath 			= getUserPersonalPath()
		self.proxy  					= ""
		self.showProgression			= 1
		self.verbose 					= 0
		self.limit						= 0
		self.defaultDir 				= os.path.join(self.userPersonalPath, u"Téléchargements")
		self.usedDir					= self.defaultDir
		self.telech						= {}
		self.execTime 					= {}
		self.downloadedOctets			= {}
		self.lasttime 					= {}
		self.downloadSpeed 				= 0
		self.timeLeft 					= "?"
		self.fileList 					= []
		self.textToShow					= ""
		self.isNewDownload				= True
		
		
		self.telechInit					= {}
		self.supportedPlatform			= {
			"Megaupload":[os.path.join(self.execDir, "icons/platform/megaupload.png"), "megaupload.com/?d=", 0, 8, r"[A-Z0-9]{8}", Megaupload],
			"Youtube":[os.path.join(self.execDir, "icons/platform/youtube.png"), "youtube.com/watch?v=", 1, 11, r"[a-zA-Z0-9_\-]{11}", Youtube],
			"Classique":[os.path.join(self.execDir, "icons/platform/classic.png"), "http://", 2, 0, r"[\w\d:#@%/;$()~_?\+-=\\\.&]*", Classic]
		}
		self.fileListByPlatform 		= {
			"Megaupload":[],
			"Youtube":[],
			"Classique":[]
		}
		self.textToShow					= {
			"Megaupload":"",
			"Youtube":"",
			"Classique":""
		}
		self.textToShow2				= {
			"Megaupload":"",
			"Youtube":"",
			"Classique":""
		}
		self.downloadSpeed 				= 0
		self.selectedPlatform = "Megaupload"
		
		
		##########################################################
		#                   Check Update/Session                 #
		##########################################################

		self.readOptions()
		
		##########################################################
		#               Arguments Var Initialisation             #
		##########################################################
		
		if("-l" in sys.argv):
			url = sys.argv[sys.argv.index("-l")+1]
			self.dwnArg = url
			if("-d" in sys.argv):
				dest = sys.argv[sys.argv.index("-d")+1]
				if(os.path.isfile(dest)):
					filename = os.path.basename(dest)
				else:
					filename = "auto"
			else:
				filename = "auto"
			self.addLink(url, filename)

		if("-ll" in sys.argv):
			listFilePath = sys.argv[sys.argv.index("-ll")+1]
			self.dwnArg = os.path.basename(listFilePath)
			if (os.path.isfile(listFilePath) and os.path.exists(listFilePath)):
				listFile = open(listFilePath, "r")
				data = listFile.read().decode("utf8")
				listFile.close()
				listLink = data.split("\n")
				if(len(listLink)>0):
					for link in listLink:
						if(link!=""):
							if ("::" in link):
								fileInfos = link.split("::")
								url = fileInfos[0]
								filename = fileInfos[1]
							else:
								url = link
								filename = "auto"
							self.addLink(url, filename)
			
			
		if("-d" in sys.argv):
			dest = sys.argv[sys.argv.index("-d")+1]
			if(os.path.isfile(dest)):
				self.usedDir = os.path.dirname(dest)
			else:
				self.usedDir = dest
	
		if("-p" in sys.argv):	
			self.showProgression = int(sys.argv[sys.argv.index("-d")+1])
			
		if("-limit" in sys.argv):
			self.limit = sys.argv[sys.argv.index("-limit")+1]
			
		
		self.depart_m()
		
	
	
	# Ajout d'un lien
	#========================================================================
	def addLink(self, url, filename, continuedl=0):
		item = "icons/wait.png"
		platformFound = False
		for downloadPlatform in self.supportedPlatform:
			if not platformFound:
				if self.supportedPlatform[downloadPlatform][1] in url:
					fileinfos = [url, filename, item, continuedl]
					self.fileListByPlatform[downloadPlatform].append(fileinfos)
					platformFound = True
			self.setTable()
	
 	# Action de lancement du téléchargement
	#========================================================================
	def depart_m(self):
		clearShell()
		print ""
		print "###############################################################################"
		print "#                                  BotMU CLI                                  #"
		print "###############################################################################"
		print "#"
		print u"# Dossier de téléchargement:"
		print "# -> ", unicode(self.usedDir)
		print "#"
		print u"# Téléchargement de:"
		print "# -> ", unicode(self.dwnArg)
		print "# "
		print "#"
	
		if len(self.fileList)>0:
			# initialisation de la barre de progression
				count=0
				for val in self.fileList:
					self.fileList[count][3] = 0
					count+=1
				# démarre le téléchargement
				destination = unicode(self.usedDir)

				if not os.path.exists(destination) or not os.path.isdir(destination):
					try:
						os.makedirs(destination)
					except:
						print "Erreur: impossible de créer le dossier de destination:\n -%s" % destination
				try:
					app = QApplication(sys.argv)
					self.telech = TelechargerMegaupload(destination, parent=self)
					self.telech.start()
					while self.telech.isRunning():
						time.sleep(1)
						pass
				except KeyboardInterrupt:
					if self.telech!=None and self.telech.isRunning():
						self.telech.stoptelech()
				except:
					print "test"
					
	def closeEvent(self, event):
		if self.telech!=None and self.telech.isRunning():
			self.telech.stoptelech()
		
	##########################################################
	#                     Thread Functions                   #
	##########################################################
				
	# Récéption et affichage d'information du téléchargement
	#========================================================================
	def infotelech(self, msg):
		"""lancé à chaque réception d'info sur la progression du téléchargement"""
		telechbloc, taillebloc, totalblocs, msgInf, row, filename = msg
		#Si la taille maxi > 0
		if totalblocs > 0:
			#Si la boucle a déjà été effectuée et que les variables sont initialisées
			if(self.lasttime!=0):
				# dès que le temps d'execution est superieur a 3s
				if(self.execTime>=3):
					#on trouve la vitesse de téléchargement
					downloadSpeedOct = int(self.downloadedOctets/3)
					self.downloadSpeed = int(downloadSpeedOct/1024)
					#et on repart a 0
					self.execTime=0.0
					self.downloadedOctets = 0
					#on peux calculer le temps restant estimé:
					#timeLeftSec = int(float(totalblocs-(telechbloc*taillebloc))/float(downloadSpeedOct))
					timeLeftSec = int(float(totalblocs-(telechbloc*taillebloc))/float(downloadSpeedOct))
					#semaine: out
					if(timeLeftSec >= 3600*24*7):
						self.timeLeft = "Plus d'une semaine"
					#jour
					elif(timeLeftSec >= 3600*24):
						timeDay = int(timeLeftSec/(3600*24))
						timeSec = timeLeftSec%(3600*24)
						timeHour = int(timeSec/3600)
						timeSec = timeLeftSec%3600
						timeMin = int(timeSec/60)
						timeSec = timeSec%60
						self.timeLeft = "%sj %sh %sm %ss" % (timeDay, timeHour, timeMin, timeSec)
					#heure
					elif(timeLeftSec >= 3600):
						timeHour = int(timeLeftSec/3600)
						timeSec = timeLeftSec%3600
						timeMin = int(timeSec/60)
						timeSec = timeSec%60
						self.timeLeft = "%sh %sm %ss" % (timeHour, timeMin, timeSec)
		
					#minutes
					elif(timeLeftSec >= 60):
						timeSec = timeLeftSec%60
						timeMin = int(timeLeftSec/60)
						self.timeLeft = "%sm %ss" % (timeMin, timeSec)
					#secondes
					else:
						self.timeLeft = "%ss" % int(timeLeftSec)
					
					p = int(float(telechbloc)*float(taillebloc)/float(totalblocs)*100)
					
					self.isNewDownload = True
					totalSize = int(totalblocs/(1024*1024))
					downloadedSize = int((telechbloc*taillebloc)/(1024*1024))
					
					textToShow = u"# %s%% -> %s/%s Mo  - %s Ko/s - ± %s" % (p, downloadedSize, totalSize, self.downloadSpeed, self.timeLeft)
					
					sys.stdout.write(chr(13))
					sys.stdout.write(' '*len(self.textToShow))
					sys.stdout.write(chr(13))
					self.textToShow = textToShow
					sys.stdout.write(self.textToShow)
					sys.stdout.flush()
		
			
				else:
					timeNow = time.time()
					timeExe = timeNow-self.lasttime
					self.downloadedOctets = self.downloadedOctets+taillebloc
					self.execTime=self.execTime+timeExe
			self.lasttime = time.time()
			
		else:
			p = 0
			timeLeft = 0
			if self.fileList[row][1]!=filename:
				self.fileList[row][1]=filename
		
		if(msgInf[1]== 1):
			if self.isNewDownload:
				self.isNewDownload = False
				print "# ** Fichier %s **\n#" % self.fileList[row][0]
			textToShow = u"# %s" % (msgInf[0])
			textToShow2 = ""
			sys.stdout.write(chr(13))
			sys.stdout.write(' '*len(self.textToShow))
			sys.stdout.write(chr(13)) 
			self.textToShow = textToShow
			sys.stdout.write(self.textToShow)
			sys.stdout.flush()
		
 
 	# Récéption et affichage d'information sur la FIN du téléchargement
	#========================================================================
	def fintelech(self, msg):
		sys.stdout.write(chr(13))
		sys.stdout.write(' '*len(self.textToShow))
		sys.stdout.write(chr(13))
		self.textToShow = ""
		sys.stdout.write("#  -> %s : 100%%\n#\n" % self.fileList[msg[1]][1])
		sys.stdout.flush()
		
	# Action de lecture des options enregistrée
	#========================================================================
	def readOptions(self):
	
		setupFile = os.path.join(self.configDir, "setup.ini")
		#s'il n'existe pas, on le créé
		if not os.path.isfile(setupFile):
			try:
				paramfile = open(setupFile, "w+")
				paramfile.close()
			except:
				logError("mainWidget - init - 1", sys.exc_info()[0], sys.exc_info()[1])
		
		paramfile = open(setupFile, "r")
		param = (paramfile.read()).split("\n")
		for item in param:
			if item.split("=")[0] == 'proxy':
				self.proxy = item.split("=")[1]
			if item.split("=")[0] == 'verbose':
				if int(item.split("=")[1]) == 1:
					self.verbose = 1
				else:
					self.verbose = 0
			if item.split("=")[0] == 'destination':
				self.defaultDir = item.split("=")[1].decode("utf8")
				self.usedDir = self.defaultDir
			if item.split("=")[0] == 'limit':
				self.limit = int(item.split("=")[1].decode("utf8").split(".")[0])
